import { ConnectStatus, CustomeSocket, SocketSettings } from "../types/socket";
import { ref } from "vue";
import { useInjector, useState } from "vue-vulcan";
import { CUSTOME_SOCKET, SOCKET_SETTINGS } from "../communication/token";

export function useSocket<S = any>() {
  const [status, setStatus]  = useState<ConnectStatus>('ready');
  const [currentMsg, setMsg] = useState<S>(null);
  const customeSocket        = useInjector<CustomeSocket<S>>(CUSTOME_SOCKET, 'optional');
  const socketSettings       = useInjector<SocketSettings>(SOCKET_SETTINGS, 'optional');

  let ws: CustomeSocket<S>;
  const brokenTime = ref(0);
  if(customeSocket) ws = customeSocket;  

  const connect = (url?: string) => {
    if(customeSocket) {
      ws.connect(url)
    } else {
      ws = new WebSocket(url) as unknown as CustomeSocket<S>
    }
    ws.addEventListener('message', event => setMsg(event.data));    
    ws.addEventListener('open', () => {
      console.log('ws链接成功')
      setStatus('opening');
      brokenTime.value = 0;
      socketSettings.heartData && startHeartConnect();
    } );    
    ws.addEventListener('close', () => {setStatus('closed'); reconnect(url);});
    ws.addEventListener('error', () => {
      setStatus('onerror'); 
      reconnect(url);
    });
  }

  // 心跳链接  
  const startHeartConnect = (data?: any) => {
    const time = socketSettings.heartInterval;
    setInterval( () => ws.send(data || socketSettings.heartData), time && time >= 3000 ? time : 5000 )    
  }
 
  // 重连功能  
  const limit = socketSettings.retryLimit || 5;
  const reconnect = (url: string) => {
    console.log('开始重连')
    brokenTime.value ++;    
    if(brokenTime.value > limit) {
      ws.close();
    } else {
      setStatus('reconnect');
      setTimeout( () => connect(url), brokenTime.value*1000 );
    }    
  }

  return {
    currentMsg,
    status,
    connect,
    startHeartConnect,
    send: ws.send,
    close: ws.close
  }
}
