import { UserData } from "../types/user";
import { useInjector, useRequest, useState } from "vue-vulcan";
import { computed, watch} from "vue";
import { useAuthData } from "../auth/useAuthData";
import { useNetSocketStore } from "any-hooks/communication/useNetSocketStore";

export function useContacts() {
  const { authData } = useInjector(useAuthData);
  const [ userList, getUserList ] = useRequest<UserData[]>('/getUserList', { auto: false });
  const { currentMsg } = useInjector(useNetSocketStore);   
  const [keyword, setKeyWord] = useState('');

  const contacts = computed( () => {
    return userList.value?.filter( item => item.id !== authData.value?.id )
                          .filter( item => item.nickname.includes(keyword.value))
  });

  const roleContacts = computed( () => {
    const contactHome = {} as {[props: string]: UserData[]}
    contacts.value?.forEach( item => {
      if(!contactHome[item.group_name]) contactHome[item.group_name] = [];
      contactHome[item.group_name].push(item);
    })
    return contactHome
  })

  const freeContacts = computed( () => contacts.value.filter( item => item.is_calling === '0' && item.is_signin === '1'))

  const getContactById = (pid: string, type?: 'uid'|'pid') => {    
    
    const index = userList.value.findIndex( user => user[type==='uid' ? 'id' : 'peer_id'] === pid );
    console.log(`根据${type  === 'uid' ?'id': 'peer_id'}${pid}查找`, index)
    return userList.value[index];
  }

  const getContactsByGroup = (groupId: string) => {
    return userList.value.filter( item => item.diy_group === groupId );
  } 

  const getContactsOfFree = () => {
    return userList.value.filter( item => item.is_calling === '0' && item.is_signin === '1');
  }

  // 监听socket的home类型消息 实时更新contacts联系人数据
  watch(currentMsg, msg => {
    if(msg.msgMainFlag !== 'Home') return;
    if(msg.msgSubFlag === 'UpdateUserList') getUserList();
  })

  return {
    contacts,
    freeContacts,
    roleContacts,
    setKeyWord,
    keyword,
    getUserList,
    getContactById,
    getContactsByGroup,
    getContactsOfFree
  }
}