import { navigateTo, redirectTo, showToast } from "@tarojs/taro";
import { useCallCenter } from "any-hooks/communication/useCallCenter";
import { watch } from "vue";
import { useInjector } from "vue-vulcan";


/** 在小程序平台监听远程联系人的呼叫动作 */
export function useCallerListener() {
  const { caller, target, myCallState } = useInjector(useCallCenter);

  watch(caller, current => {      
    if(!current) return;  
    switch(current.action) {
      case 'Request':        
      break;
      case 'Hangup':          
        showToast({title: '对方取消了呼叫', icon: 'none'});
      break;
    }  
  })

  watch(target, val => {
    switch(val?.action) {
      case 'Busying':
        showToast({title: '对方繁忙，暂时无法接受呼叫', icon: 'none'});
      break;
      case 'Hangup':
        showToast({title: '对方拒绝了您的呼叫', icon: 'none'});
      break;
    }
  })

  // 根据用户的呼叫状态变化，执行相应的页面跳转逻辑
  watch(myCallState, (state, prev) => {
    switch(state) {
      case 'calling':
        if(prev !== 'free') {
          showToast({title:'已发出呼叫，等待对方回应...', icon:'none'});          
        }else {
          navigateTo({url: '/pages/calling/index'});
        }        
      break;
      case 'being_called':
        navigateTo({url: '/pages/calling/index'});
      break;
      case 'call_accepted':        
      case 'call_successed':    
        redirectTo({url: '/pages/meeting/index'});
      break;    
    }
  })
}