import { navigateTo, showModal } from "@tarojs/taro";
import { watch } from "@vue/runtime-core";
import { useAuthData } from "any-hooks/auth/useAuthData";
import { useLogOut } from "any-hooks/auth/useLogOut";
import { useNetSocketStore } from "any-hooks/communication/useNetSocketStore";
import { useInjector, useRequest } from "vue-vulcan";

export function useSocketHandle() {
  const { status, initConnect } = useInjector(useNetSocketStore); 
  const { authData } = useInjector(useAuthData);
  const [,requestAuthState] = useRequest<{is_signin: '1' | '0'}>('/getUserCallStatus', {auto: false});
  const { submitLogOut } = useLogOut();

  watch(authData, data => {    
    if(!data) return;
    requestAuthState({user_id: data.id})
			.then(res => {
        if(res.is_signin === '0') {
          initConnect(data)
        } else {
          showModal({
            title:'当前账号已在线，是否确认登陆?',
            success: e => {
              if(e.confirm){
                initConnect(data);                
              } else if(e.cancel){
                submitLogOut({login_id: data.id}).then(_ => navigateTo({url: '/pages/login/index'}));
              }
            }
          })
        }        
      })    
  })
  

  watch(status, val => {
    console.log('socket status', val)
  })
}